"""In-process tests for the modulemd generator"""

from click.testing import CliRunner
import gi
gi.require_version('Modulemd', '1.0')  # noqa: E402
from gi.repository import Modulemd
import pytest

from _fedmod.cli import _cli_commands


def _generate_modulemd(rpms):
    cmd = ['rpm2module']
    cmd.extend(rpms)
    runner = CliRunner()
    result = runner.invoke(_cli_commands, cmd)
    assert result.exit_code == 0
    modmd = Modulemd.Module.new_from_string(result.output)
    return modmd


class TestSinglePackageInput(object):

    @pytest.mark.needs_metadata
    def test_generated_modulemd_file(self):
        input_rpms = ('grep',)
        modmd = _generate_modulemd(input_rpms)

        # Expected description for 'grep'
        assert modmd.props.summary == "Pattern matching utilities"
        assert modmd.props.description.startswith(
            "The GNU versions of commonly used grep utilities")

        # Expected licenses for 'grep'
        assert modmd.props.module_licenses.get() == ['MIT']
        assert modmd.props.content_licenses.get() == []

        # Only given modules are listed in the public API
        assert sorted(modmd.props.rpm_api.get()) == sorted(input_rpms)

        # Expected components for grep
        assert set(modmd.props.components_rpm) == set(input_rpms)

        # Expected module dependencies for grep
        dependencies = modmd.props.dependencies
        assert len(dependencies) == 1

        buildrequires = dependencies[0].props.buildrequires
        assert set(buildrequires) == {'platform'}
        assert buildrequires['platform'].get() == []

        requires = dependencies[0].props.requires
        assert set(requires) == {'platform'}
        assert requires['platform'].get() == []


class TestMultiplePackageInput(object):

    @pytest.mark.needs_metadata
    def test_generated_modulemd_file(self):
        input_rpms = ('grep', 'haproxy')
        modmd = _generate_modulemd(input_rpms)

        # Expected description for 'grep'
        assert modmd.props.summary == "Pattern matching utilities"
        assert modmd.props.description.startswith(
            "The GNU versions of commonly used grep utilities")

        # Expected licenses for grep + haproxy
        assert modmd.props.module_licenses.get() == ['MIT']
        # This doesn't seem right...
        assert modmd.props.content_licenses.get() == []

        # Only given modules are listed in the public API
        assert sorted(modmd.props.rpm_api.get()) == sorted(input_rpms)

        # Expected components for grep + haproxy
        expected_components = set(input_rpms)
        assert set(modmd.props.components_rpm) == expected_components

        assert modmd.props.components_rpm['grep'].props.rationale == "Package in api"

        # Expected module dependencies for grep
        dependencies = modmd.props.dependencies
        assert len(dependencies) == 1

        buildrequires = dependencies[0].props.buildrequires
        assert set(buildrequires) == {'platform'}
        assert buildrequires['platform'].get() == []

        requires = dependencies[0].props.requires
        assert set(requires) == {'platform'}
        assert requires['platform'].get() == []
